package com.example.React_Native_back.board.controller;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import com.example.React_Native_back.board.dto.BoardDTO;
import com.example.React_Native_back.board.dto.ImageDTO;
import com.example.React_Native_back.board.service.BoardService;
import com.example.React_Native_back.board.service.FileService;
import com.example.React_Native_back.utils.ApiResponse;

import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.Value;
import lombok.extern.log4j.Log4j2;

@RestController
@RequestMapping("/api")
@RequiredArgsConstructor
public class BoardController {

	private final BoardService boardService;
	private final FileService fileService;
	
	@GetMapping("/getList")
	public ApiResponse<List<BoardDTO>>  getList() throws Exception{
		System.out.println("controller getList :: " + boardService);
//		boardService.getList();
		return ApiResponse.successResponse(boardService.getList());
	}
	
	@PostMapping("/updateAtoB")
	public int updateAtoB(@RequestBody BoardDTO boardDTO) throws Exception{
		System.out.println("boardDTO controller updateAtoB : " + boardDTO);
		if(boardDTO != null) {
			return boardService.updateAtoB(boardDTO);	
		}
		return -1;
	}

	@PostMapping("/getDetail")
	public BoardDTO getDetail(@RequestBody String boardSeq) throws Exception{
		System.out.println("boardSeq controller getDetail : " + boardSeq);
		if(boardSeq != null) {
			return boardService.getDetail(boardSeq);	
		}
		return null;
	}
	
	@PostMapping("/saveImage")
    public ApiResponse<?> saveImage(@ModelAttribute MultipartFile file) throws Exception{
		System.err.println("file controller saveImage : " + file);

		if (file == null || file.isEmpty()) {
			System.out.println("No file uploaded.");
	        return ApiResponse.errorResponse("No file uploaded.") ;
	        		//ResponseEntity.badRequest().body("No file uploaded.");
	    }
	    if (!file.getContentType().startsWith("image/")) {
			System.out.println("Invalid file type. Only images are allowed..");
	        return ApiResponse.errorResponse("Invalid file type. Only images are allowed.") ;
	        //ResponseEntity.badRequest().body("Invalid file type. Only images are allowed.");
	    }

	    try {
	    	fileService.saveImage(file);	
	    	System.out.println("try");
	        return ApiResponse.successResponse("file uploaded.") ;
	    } catch (Exception e) {
			System.out.println("catch : " + e);
	        return ApiResponse.errorResponse("Image upload failed: " + e.getMessage()) ;
	        //ResponseEntity.status(500).body("Image upload failed: " + e.getMessage());
	    }
    }	
	@GetMapping("/getImageId")
	public ApiResponse<?>  getImageId() throws Exception{
		System.out.println("controller getImageId :: " + boardService);
//		boardService.getList();
		return ApiResponse.successResponse(fileService.getImageId());
	}
	
//	@PostMapping("/getFile")
//	public ApiResponse<byte[]> getFile(String boardImage) throws Exception{
//		System.out.println("controller getFile :: " + boardImage );
//		// 파일 읽어오기
//        byte[] fileContent = readFileContent(boardImage);
//        // 파일 인코딩
//        String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8);
//        // 파일 다운로드 응답
//        return createResponseEntity(encodedFileName, fileContent);
//		return ApiResponse.successResponse(fileService.getImageId());
//	}

}
  